@@ -1,4 +1,4 @@
-# Copyright 2012-2013 Arthur Noel
+# Copyright 2012-2014 Arthur Noel
 #
 # This file is part of Stacked.
 #
@@ -14,6 +14,8 @@
 # You should have received a copy of the GNU General Public License along with
 # Stacked. If not, see <http://www.gnu.org/licenses/>.
 
+import copy
+
 
 class Stacked(object):
 
@@ -38,41 +40,38 @@ class Stacked(object):
             current = getattr(target, member, None)
             if isinstance(current, dict) and isinstance(patch, dict):
                 # if we are patching a dictionary we want to update not replace
-                for key in current:
-                    if key not in patch:
-                        patch[key] = current[key]
+                tmp = copy.copy(current)
+                tmp.update(patch)
+                patch = tmp
         self._patch_target(target, member, patch)
         self._patch_stack.append((target, member, current))
 
-    def _register_patch(self, target, member, patch):
+    def register_patch(self, target, member, patch):
         self._to_patch.append((target, member, patch))
 
     def _push(self, obj):
         self._push_stack.append(obj)
         obj.__enter__()
 
-    def _register_push(self, obj):
+    def register_push(self, obj):
         self._to_push.append(obj)
 
-    def _register_push_member(self, name, obj):
+    def register_push_member(self, name, obj):
         setattr(self, name, obj)
         self._to_push_member.append(name)
 
     def __enter__(self):
-        if self._entered:
-            raise RuntimeError("Can't reenter %r " % self)
-        self._entered = True
-        for patch in self._to_patch:
-            self._patch(*patch)
-        for obj in self._to_push:
-            self._push(obj)
-        for name in self._to_push_member:
-            self._push(getattr(self, name))
-        return self
-
-    def __exit__(self, etype, evalue, etraceback):
         if not self._entered:
-            raise RuntimeError("Haven't entered %r" % self)
+            self._entered = True
+            for patch in self._to_patch:
+                self._patch(*patch)
+            for obj in self._to_push:
+                self._push(obj)
+            for name in self._to_push_member:
+                self._push(getattr(self, name))
+            return self
+
+    def __exit__(self, etype=None, evalue=None, etraceback=None):
         while self._push_stack:
             obj = self._push_stack.pop()
             obj.__exit__(etype, evalue, etraceback)
