@@ -1,4 +1,4 @@
-# Copyright 2012-2013 Arthur Noel
+# Copyright 2012-2014 Arthur Noel
 #
 # This file is part of Stacked.
 #
@@ -65,7 +65,7 @@ class TestStacked(unittest.TestCase):
         stacked = Stacked()
         obj = _TestObject()
         to_patch = [(obj, "one", 2)]
-        stacked._register_patch(obj, "one", 2)
+        stacked.register_patch(obj, "one", 2)
         self.assertEqual(obj.one, 1)
         self.assertEqual(stacked._to_patch, to_patch)
         with stacked:
@@ -82,7 +82,7 @@ class TestStacked(unittest.TestCase):
     def test_register_push(self):
         stacked = Stacked()
         obj = _TestStackedObject()
-        stacked._register_push(obj)
+        stacked.register_push(obj)
         self.assertFalse(obj._entered)
         self.assertEqual(stacked._push_stack, [])
         with stacked:
@@ -94,7 +94,7 @@ class TestStacked(unittest.TestCase):
     def test_register_push_member(self):
         stacked = Stacked()
         obj = _TestStackedObject()
-        stacked._register_push_member("obj", obj)
+        stacked.register_push_member("obj", obj)
         self.assertEqual(stacked.obj, obj)
         self.assertFalse(obj._entered)
         self.assertEqual(stacked._push_stack, [])
@@ -104,11 +104,14 @@ class TestStacked(unittest.TestCase):
         self.assertFalse(obj._entered)
         self.assertEqual(stacked._push_stack, [])
 
-    def test_double_entry(self):
+    def test_entry(self):
         stacked = Stacked()
         stacked.__enter__()
-        self.assertRaises(RuntimeError, stacked.__enter__)
-
-    def test_bad_exit(self):
-        stacked = Stacked()
-        self.assertRaises(RuntimeError, stacked.__exit__, None, None, None)
+        self.assertTrue(stacked._entered)
+        # double entry is a noop
+        stacked.__enter__()
+        self.assertTrue(stacked._entered)
+        stacked.__exit__(None, None, None)
+        self.assertFalse(stacked._entered)
+        stacked.__exit__(None, None, None)
+        self.assertFalse(stacked._entered)
